@@ -3,6 +3,7 @@
 use crate::ast;
 use crate::ast::analyze_deno_types;
 use crate::ast::analyze_dependencies;
+use crate::ast::analyze_jsx_import_sources;
 use crate::ast::analyze_ts_references;
 use crate::ast::SourceParser;
 use crate::module_specifier::resolve_import;
@@ -663,8 +664,8 @@ impl ModuleGraph {
   ) -> Option<&ModuleSpecifier> {
     let referrer = self.resolve(referrer);
     let referring_module_slot = self.module_slots.get(&referrer)?;
-    let maybe_specifier =
-      if let ModuleSlot::Module(referring_module) = referring_module_slot {
+    let maybe_specifier = match referring_module_slot {
+      ModuleSlot::Module(referring_module) => {
         let dependency = referring_module.dependencies.get(specifier)?;
         let (maybe_first, maybe_second) = if prefer_types {
           (&dependency.maybe_type, &dependency.maybe_code)
@@ -686,18 +687,31 @@ impl ModuleGraph {
         } else {
           None
         }
-      } else {
-        None
-      };
+      }
+      ModuleSlot::SyntheticModule(referring_module) => {
+        match referring_module.dependencies.get(specifier) {
+          Some(Some(Ok((specifier, _)))) => {
+            if prefer_types {
+              Some(
+                self
+                  .resolve_types_dependency(specifier)
+                  .unwrap_or(specifier),
+              )
+            } else {
+              Some(specifier)
+            }
+          }
+          _ => None,
+        }
+      }
+      _ => return None,
+    };
     // Even if we resolved the specifier, it doesn't mean the module is actually
     // there, and so we will return the final final specifier.
-    if let Some(specifier) = maybe_specifier {
-      match self.module_slots.get(&self.resolve(specifier)) {
-        Some(ModuleSlot::Module(m)) => Some(&m.specifier),
-        _ => None,
-      }
-    } else {
-      None
+    let specifier = maybe_specifier?;
+    match self.module_slots.get(&self.resolve(specifier)) {
+      Some(ModuleSlot::Module(m)) => Some(&m.specifier),
+      _ => None,
     }
   }
 
@@ -1017,6 +1031,19 @@ pub(crate) fn parse_module_from_ast(
     }
   }
 
+  // Analyze any JSX Import Source pragma
+  if let Some((import_source, span)) = analyze_jsx_import_sources(parsed_source)
+  {
+    let jsx_import_source_module = maybe_resolver
+      .map(|r| r.jsx_import_source_module())
+      .unwrap_or(DEFAULT_JSX_IMPORT_SOURCE_MODULE);
+    let specifier = format!("{}/{}", import_source, jsx_import_source_module);
+    let range = Range::from_swc_span(&module.specifier, parsed_source, &span);
+    let resolved_dependency = resolve(&specifier, &range, maybe_resolver);
+    let dep = module.dependencies.entry(specifier).or_default();
+    dep.maybe_code = resolved_dependency;
+  }
+
   // Analyze the X-TypeScript-Types header
   if module.maybe_types_dependency.is_none() {
     if let Some(headers) = maybe_headers {
@@ -1131,11 +1158,12 @@ fn is_untyped(media_type: &MediaType) -> bool {
 }
 
 pub(crate) struct Builder<'a> {
-  is_dynamic_root: bool,
+  in_dynamic_branch: bool,
   graph: ModuleGraph,
   loader: &'a mut dyn Loader,
   maybe_resolver: Option<&'a dyn Resolver>,
   pending: FuturesUnordered<LoadFuture>,
+  dynamic_branches: HashSet<ModuleSpecifier>,
   source_parser: &'a dyn SourceParser,
 }
 
@@ -1149,11 +1177,12 @@ impl<'a> Builder<'a> {
     source_parser: &'a dyn SourceParser,
   ) -> Self {
     Self {
-      is_dynamic_root,
+      in_dynamic_branch: is_dynamic_root,
       graph: ModuleGraph::new(roots, maybe_locker),
       loader,
       maybe_resolver,
       pending: FuturesUnordered::new(),
+      dynamic_branches: HashSet::new(),
       source_parser,
     }
   }
@@ -1164,7 +1193,7 @@ impl<'a> Builder<'a> {
   ) -> ModuleGraph {
     let roots = self.graph.roots.clone();
     for root in roots {
-      self.load(&root, self.is_dynamic_root);
+      self.load(&root, self.in_dynamic_branch);
     }
 
     // Process any imports that are being added to the graph.
@@ -1178,7 +1207,7 @@ impl<'a> Builder<'a> {
         for (specifier, _) in
           synthetic_module.dependencies.values().flatten().flatten()
         {
-          self.load(specifier, self.is_dynamic_root);
+          self.load(specifier, self.in_dynamic_branch);
         }
         self
           .graph
@@ -1210,7 +1239,23 @@ impl<'a> Builder<'a> {
         _ => {}
       }
       if self.pending.is_empty() {
-        break;
+        // Start visiting queued up dynamic branches. We do this in a separate
+        // pass after all static dependencies have been visited because:
+        // - If a module is both statically and dynamically imported, we want
+        //   the static import to take precedence and only load it with
+        //   `is_dynamic: false`.
+        // - It's more convenient for tracking whether or not we are currently
+        //   visiting a dynamic branch.
+        if !self.in_dynamic_branch {
+          self.in_dynamic_branch = true;
+          for specifier in std::mem::take(&mut self.dynamic_branches) {
+            if !self.graph.module_slots.contains_key(&specifier) {
+              self.load(&specifier, true);
+            }
+          }
+        } else {
+          break;
+        }
       }
     }
 
@@ -1274,11 +1319,22 @@ impl<'a> Builder<'a> {
 
     if let ModuleSlot::Module(module) = &module_slot {
       for dep in module.dependencies.values() {
-        if let Some(Ok((specifier, _))) = &dep.maybe_code {
-          self.load(specifier, dep.is_dynamic);
-        }
-        if let Some(Ok((specifier, _))) = &dep.maybe_type {
-          self.load(specifier, dep.is_dynamic);
+        // Queue up dynamic dependencies to be visited later, unless we are
+        // already visiting a dynamic branch.
+        if dep.is_dynamic && !self.in_dynamic_branch {
+          if let Some(Ok((specifier, _))) = &dep.maybe_code {
+            self.dynamic_branches.insert(specifier.clone());
+          }
+          if let Some(Ok((specifier, _))) = &dep.maybe_type {
+            self.dynamic_branches.insert(specifier.clone());
+          }
+        } else {
+          if let Some(Ok((specifier, _))) = &dep.maybe_code {
+            self.load(specifier, self.in_dynamic_branch);
+          }
+          if let Some(Ok((specifier, _))) = &dep.maybe_type {
+            self.load(specifier, self.in_dynamic_branch);
+          }
         }
       }
 
@@ -1426,6 +1482,8 @@ where
 #[cfg(test)]
 mod tests {
   use super::*;
+  use crate::ast::DefaultSourceParser;
+  use url::Url;
 
   #[test]
   fn test_range_includes() {
@@ -1522,4 +1580,87 @@ mod tests {
       panic!("no module returned");
     }
   }
+
+  #[tokio::test]
+  async fn static_dep_of_dynamic_dep_is_dynamic() {
+    struct TestLoader {
+      loaded_foo: bool,
+      loaded_bar: bool,
+      loaded_baz: bool,
+    }
+    impl Loader for TestLoader {
+      fn load(
+        &mut self,
+        specifier: &ModuleSpecifier,
+        is_dynamic: bool,
+      ) -> LoadFuture {
+        let specifier = specifier.clone();
+        match specifier.as_str() {
+          "file:///foo.js" => {
+            assert!(!is_dynamic);
+            self.loaded_foo = true;
+            Box::pin(async move {
+              (
+                specifier.clone(),
+                Ok(Some(LoadResponse {
+                  specifier: specifier.clone(),
+                  maybe_headers: None,
+                  content: Arc::new(
+                    "await import('file:///bar.js')".to_string(),
+                  ),
+                })),
+              )
+            })
+          }
+          "file:///bar.js" => {
+            assert!(is_dynamic);
+            self.loaded_bar = true;
+            Box::pin(async move {
+              (
+                specifier.clone(),
+                Ok(Some(LoadResponse {
+                  specifier: specifier.clone(),
+                  maybe_headers: None,
+                  content: Arc::new("import 'file:///baz.js'".to_string()),
+                })),
+              )
+            })
+          }
+          "file:///baz.js" => {
+            assert!(is_dynamic);
+            self.loaded_baz = true;
+            Box::pin(async move {
+              (
+                specifier.clone(),
+                Ok(Some(LoadResponse {
+                  specifier: specifier.clone(),
+                  maybe_headers: None,
+                  content: Arc::new("console.log('Hello, world!')".to_string()),
+                })),
+              )
+            })
+          }
+          _ => unreachable!(),
+        }
+      }
+    }
+    let mut loader = TestLoader {
+      loaded_foo: false,
+      loaded_bar: false,
+      loaded_baz: false,
+    };
+    let source_parser = DefaultSourceParser::new();
+    let builder = Builder::new(
+      vec![Url::parse("file:///foo.js").unwrap()],
+      false,
+      &mut loader,
+      None,
+      None,
+      &source_parser,
+    );
+    builder.build(None).await;
+    assert!(loader.loaded_foo);
+    assert!(loader.loaded_bar);
+    assert!(loader.loaded_baz);
+  }
 }
