@@ -127,6 +127,7 @@ cfg_if! {
     pub async fn js_create_graph(
       roots: JsValue,
       load: js_sys::Function,
+      maybe_jsx_import_source_module: Option<String>,
       maybe_cache_info: Option<js_sys::Function>,
       maybe_resolve: Option<js_sys::Function>,
       maybe_resolve_types: Option<js_sys::Function>,
@@ -138,8 +139,8 @@ cfg_if! {
       let roots_vec: Vec<String> = roots.into_serde().map_err(|err| JsValue::from(js_sys::Error::new(&err.to_string())))?;
       let maybe_imports_map: Option<HashMap<String, Vec<String>>> = maybe_imports.into_serde().map_err(|err| JsValue::from(js_sys::Error::new(&err.to_string())))?;
       let mut loader = js_graph::JsLoader::new(load, maybe_cache_info);
-      let maybe_resolver = if maybe_resolve.is_some() || maybe_resolve_types.is_some() {
-        Some(js_graph::JsResolver::new(maybe_resolve, maybe_resolve_types))
+      let maybe_resolver = if maybe_jsx_import_source_module.is_some() || maybe_resolve.is_some() || maybe_resolve_types.is_some() {
+        Some(js_graph::JsResolver::new(maybe_jsx_import_source_module, maybe_resolve, maybe_resolve_types))
       } else {
         None
       };
@@ -184,6 +185,7 @@ cfg_if! {
     pub fn js_parse_module(
       specifier: String,
       maybe_headers: JsValue,
+      maybe_jsx_import_source_module: Option<String>,
       content: String,
       maybe_resolve: Option<js_sys::Function>,
       maybe_resolve_types: Option<js_sys::Function>,
@@ -193,8 +195,8 @@ cfg_if! {
         .map_err(|err| js_sys::Error::new(&err.to_string()))?;
       let specifier = module_specifier::ModuleSpecifier::parse(&specifier)
         .map_err(|err| js_sys::Error::new(&err.to_string()))?;
-      let maybe_resolver = if maybe_resolve.is_some() || maybe_resolve_types.is_some() {
-        Some(js_graph::JsResolver::new(maybe_resolve, maybe_resolve_types))
+      let maybe_resolver = if maybe_jsx_import_source_module.is_some() || maybe_resolve.is_some() || maybe_resolve_types.is_some() {
+        Some(js_graph::JsResolver::new(maybe_jsx_import_source_module, maybe_resolve, maybe_resolve_types))
       } else {
         None
       };
@@ -527,6 +529,74 @@ console.log(a);
     );
   }
 
+  #[tokio::test]
+  async fn test_create_graph_imports_resolve_dependency() {
+    let mut loader = setup(
+      vec![
+        (
+          "file:///a/test01.ts",
+          Ok(("file:///a/test01.ts", None, r#"console.log("a");"#)),
+        ),
+        (
+          "https://example.com/jsx-runtime",
+          Ok((
+            "https://example.com/jsx-runtime",
+            Some(vec![
+              ("content-type", "application/javascript"),
+              ("x-typescript-types", "./jsx-runtime.d.ts"),
+            ]),
+            r#"export const a = "a";"#,
+          )),
+        ),
+        (
+          "https://example.com/jsx-runtime.d.ts",
+          Ok((
+            "https://example.com/jsx-runtime.d.ts",
+            Some(vec![("content-type", "application/typescript")]),
+            r#"export const a: "a";"#,
+          )),
+        ),
+      ],
+      vec![],
+    );
+    let root_specifier = ModuleSpecifier::parse("file:///a/test01.ts").unwrap();
+    let config_specifier =
+      ModuleSpecifier::parse("file:///a/tsconfig.json").unwrap();
+    let maybe_imports = Some(vec![(
+      config_specifier.clone(),
+      vec!["https://example.com/jsx-runtime".to_string()],
+    )]);
+    let graph = create_graph(
+      vec![root_specifier],
+      false,
+      maybe_imports,
+      &mut loader,
+      None,
+      None,
+      None,
+    )
+    .await;
+    assert_eq!(
+      graph.resolve_dependency(
+        "https://example.com/jsx-runtime",
+        &config_specifier,
+        false
+      ),
+      Some(&ModuleSpecifier::parse("https://example.com/jsx-runtime").unwrap())
+    );
+    assert_eq!(
+      graph.resolve_dependency(
+        "https://example.com/jsx-runtime",
+        &config_specifier,
+        true
+      ),
+      Some(
+        &ModuleSpecifier::parse("https://example.com/jsx-runtime.d.ts")
+          .unwrap()
+      )
+    );
+  }
+
   #[tokio::test]
   async fn test_create_graph_with_headers() {
     let mut loader = setup(
@@ -567,6 +637,90 @@ console.log(a);
     }
   }
 
+  #[tokio::test]
+  async fn test_create_graph_jsx_import_source() {
+    let mut loader = setup(
+      vec![
+        (
+          "file:///a/test01.tsx",
+          Ok((
+            "file:///a/test01.tsx",
+            None,
+            r#"/* @jsxImportSource https://example.com/preact */
+            
+            export function A() {
+              <div>Hello Deno</div>
+            }
+            "#,
+          )),
+        ),
+        (
+          "https://example.com/preact/jsx-runtime",
+          Ok((
+            "https://example.com/preact/jsx-runtime/index.js",
+            Some(vec![("content-type", "application/javascript")]),
+            r#"export function jsx() {}"#,
+          )),
+        ),
+      ],
+      vec![],
+    );
+    let root_specifier =
+      ModuleSpecifier::parse("file:///a/test01.tsx").expect("bad url");
+    let graph = create_graph(
+      vec![root_specifier.clone()],
+      false,
+      None,
+      &mut loader,
+      None,
+      None,
+      None,
+    )
+    .await;
+    assert_eq!(
+      json!(graph),
+      json!({
+        "roots": [
+          "file:///a/test01.tsx"
+        ],
+        "modules": [
+          {
+            "dependencies": [
+              {
+                "specifier": "https://example.com/preact/jsx-runtime",
+                "code": {
+                  "specifier": "https://example.com/preact/jsx-runtime",
+                  "span": {
+                    "start": {
+                      "line": 0,
+                      "character": 20
+                    },
+                    "end": {
+                      "line": 0,
+                      "character": 46
+                    }
+                  }
+                }
+              }
+            ],
+            "mediaType": "TSX",
+            "size": 159,
+            "specifier": "file:///a/test01.tsx"
+          },
+          {
+            "dependencies": [],
+            "mediaType": "JavaScript",
+            "size": 24,
+            "specifier": "https://example.com/preact/jsx-runtime/index.js"
+          }
+        ],
+        "redirects": {
+          "https://example.com/preact/jsx-runtime": "https://example.com/preact/jsx-runtime/index.js"
+        }
+      })
+    );
+  }
+
   #[tokio::test]
   async fn test_bare_specifier_error() {
     let mut loader = setup(
@@ -1080,6 +1234,45 @@ console.log(a);
     assert_eq!(actual.media_type, MediaType::TypeScript);
   }
 
+  #[test]
+  fn test_parse_module_jsx_import_source() {
+    let specifier = ModuleSpecifier::parse("file:///a/test01.tsx").unwrap();
+    let result = parse_module(
+      &specifier,
+      None,
+      Arc::new(
+        r#"
+    /** @jsxImportSource https://example.com/preact */
+
+    export function A() {
+      return <div>Hello Deno</div>;
+    }
+    "#
+        .to_string(),
+      ),
+      None,
+      None,
+    );
+    assert!(result.is_ok());
+    let actual = result.unwrap();
+    assert_eq!(actual.dependencies.len(), 1);
+    let dep = actual
+      .dependencies
+      .get("https://example.com/preact/jsx-runtime")
+      .unwrap();
+    assert!(dep.maybe_code.is_some());
+    let code_dep = dep.maybe_code.clone().unwrap();
+    assert!(code_dep.is_ok());
+    let (dep_specifier, _) = code_dep.unwrap();
+    assert_eq!(
+      dep_specifier,
+      ModuleSpecifier::parse("https://example.com/preact/jsx-runtime").unwrap()
+    );
+    assert!(dep.maybe_type.is_none());
+    assert_eq!(actual.specifier, specifier);
+    assert_eq!(actual.media_type, MediaType::Tsx);
+  }
+
   #[test]
   fn test_parse_module_with_headers() {
     let specifier = ModuleSpecifier::parse("https://localhost/file").unwrap();
